﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using SharpDX;

namespace PackingPlanVisualizations {

  public class CenteredContainer {
    public BasicCuboidShape Container { get; private set; }
    public List<BasicCuboidShape> PackingItems { get; private set; }

    private readonly float normalizingFactor;

    public CenteredContainer(BasicCuboidShape container) {
      normalizingFactor = CalculateNormalizingFactor(container);
      Container = Normalize(container);
      PackingItems = new List<BasicCuboidShape>();
    }

    private float CalculateNormalizingFactor(BasicCuboidShape shape) {
      float longestEdge = Math.Max(Math.Max(shape.ShapeSize.X, shape.ShapeSize.Y), shape.ShapeSize.Z);
      return 1 / (longestEdge / 30);
    }
    private BasicCuboidShape Normalize(BasicCuboidShape shape) {
      return new BasicCuboidShape(shape.ShapeSize * normalizingFactor, shape.ShapePosition * normalizingFactor, shape.ShapeID, shape.Material);
    }

    public void AddPackingItem(Vector3 size, Vector3 position, int index, int material) {
      var itemSize = size * normalizingFactor;
      var itemPosition = position * normalizingFactor;
      var actualPosition = Container.CalculatePositionRelativeToBottomLeftBackCorner(itemSize, itemPosition);
      var itemShape = new BasicCuboidShape(itemSize, actualPosition, index, material);
      PackingItems.Add(itemShape);
    }
  }
}